/*
 *  stereo -- Stereo matching software
 *  $Id: unnormalise.c 228 2004-08-16 07:25:05Z eric $
 *  Copyright (c) 2004 Sam Hocevar <sam@zoy.org>
 *                     Éric Beets <ericbeets@free.fr>
 *  All Rights Reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA
 */

#include <stdint.h>

#include "defs.h"
#include "main.h"
#include "geometry.h"

static pt2d_t transform(projection_t *data,
                        pt2d_t from);

/* Unnormalise depth map */
int unnormalise(image_t *image,
                projection_t *data,
                image_t *dest)
{
    int x, y;
    int r,g,b;
    /* XXX: Naive unnormalise */
    for(y = 0; y < dest->height; y++)
        for(x = 0; x < dest->width; x++)
        {
            pt2d_t from, to;

            from.x = x;
            from.y = y;
            to = transform(data, from);

           //dest->pixels[y * dest->pitch + x] = image->pixels[(int)(to.y - data->top) * image->pitch + (int)(to.x - data->left)];
	    getpixel(image,(int)(to.x-data->left), (int)(to.y-data->top), &r, &g, &b);
	    setpixel(dest,x,y,r,g,b);
        }

    return 0;
}

static pt2d_t transform(projection_t *data,
                        pt2d_t from)
{
    pt3d_t pt3d;
    pt2d_t to;

    pt3d.x = from.x;
    pt3d.y = from.y;
    pt3d.z = data->d;

    pt3d = multiply(1 / scalar_product(pt3d, data->Os), pt3d);
    pt3d = multiply(scalar_product(data->Os, data->Os), pt3d);

    to.x = scalar_product(pt3d, data->vx);
    to.y = scalar_product(pt3d, data->vy);

    return to;
}

